cordova.define("cordova-plugin-ucaudio.UCAudioPlugin", function(require, exports, module) {
               
               var argscheck = require('cordova/argscheck'),
               utils = require('cordova/utils'),
               exec = require('cordova/exec');

var mediaObjects = {};

var UCAudioPlugin = function (successCallback, errorCallback, statusCallback) {
    this.id = utils.createUUID();
    mediaObjects[this.id] = this;
    this.successCallback = successCallback;
    this.errorCallback = errorCallback;
    this.statusCallback = statusCallback;
    this._duration = -1;
    this._position = -1;
    this._isPlaying = false;
    exec(null, this.errorCallback, "UCAudioPlugin", "create", [this.id]);
};

UCAudioPlugin.get = function(id) {
    return mediaObjects[id];
};


UCAudioPlugin.prototype.prepare = function (url, interval) {
    exec(this.successCallback, this.errorCallback, "UCAudioPlugin", "prepare", [url, interval]);
};

UCAudioPlugin.prototype.play = function () {
   exec(this.successCallback, this.errorCallback, "UCAudioPlugin", "play", []);
};

UCAudioPlugin.prototype.pause = function () {
   exec(this.successCallback, this.errorCallback, "UCAudioPlugin", "pause", []);
};

UCAudioPlugin.prototype.stop = function () {
   exec(this.successCallback, this.errorCallback, "UCAudioPlugin", "stop", []);
};

UCAudioPlugin.prototype.seekTo = function (position) {
    exec(this.successCallback, this.errorCallback, "UCAudioPlugin", "seekTo", [position]);
};

UCAudioPlugin.prototype.setSpeed = function (speed) {
    exec(this.successCallback, this.errorCallback, "UCAudioPlugin", "setSpeed", [speed]);
};

UCAudioPlugin.prototype.setVolume = function (volume) {
    exec(this.successCallback, this.errorCallback, "UCAudioPlugin", "setVolume", [volume]);
};

UCAudioPlugin.prototype.release = function () {
   exec(this.successCallback, this.errorCallback, "UCAudioPlugin", "release", []);
};

UCAudioPlugin.prototype.isPlaying = function() {
    return this._isPlaying;
};

UCAudioPlugin.prototype.getDuration = function() {
    return this._duration;
};

UCAudioPlugin.prototype.getCurrentDuration = function() {
    return this._position;
};

    UCAudioPlugin.STATE_IDLE = -1;
    UCAudioPlugin.STATE_NORMAL = 0;
    UCAudioPlugin.STATE_PREPARING = 1;
    UCAudioPlugin.STATE_PREPARING_PLAYING = 2;
    UCAudioPlugin.STATE_PREPARED = 3;
    UCAudioPlugin.STATE_PLAYING = 4;
    UCAudioPlugin.STATE_PROGRESS = 5;
    UCAudioPlugin.STATE_PAUSE = 6;
    UCAudioPlugin.STATE_SEEKPROCESSED = 7;
    UCAudioPlugin.STATE_AUTOCOMPLETE = 8;
    UCAudioPlugin.STATE_ERROR = 9;

UCAudioPlugin.onStatus = function(id, msgType, value) {
    var media = mediaObjects[id];

    console.log("send state :: " + msgType + "value:" +value);
    if (media) {

        switch(msgType) {
            case UCAudioPlugin.STATE_PROGRESS :
            case UCAudioPlugin.STATE_SEEKPROCESSED:
                media._position = value;
            break;

            case UCAudioPlugin.STATE_PREPARED :
                media._duration = value;
                media._isPlaying = false;
                break;

            case UCAudioPlugin.STATE_PLAYING :
                media._duration = value;
                media._isPlaying = true;
                break;
            case UCAudioPlugin.STATE_PAUSE :
                media._duration = value;
                media._isPlaying = false;
                break;
            case UCVideoPlugin.STATE_ERROR :
            case UCVideoPlugin.STATE_AUTOCOMPLETE :
                media._isPlaying = false;
                break;

            default :
            break;
        }

        var msg = {"type": msgType, "value": value};
        media.statusCallback(msg);
    } else if (console.error) {
        console.error("Received UCAudioPlugin.onStatus callback for unknown media :: " + id);
    }

};

    module.exports = UCAudioPlugin;

    function onMessageFromNative(msg) {
        if (msg.action == 'status') {
            UCAudioPlugin.onStatus(msg.status.id, msg.status.msgType, msg.status.value);
        } else {
            throw new Error('Unknown media action' + msg.action);
        }
    }

    if (cordova.platformId === 'android' || cordova.platformId === 'amazon-fireos' || cordova.platformId === 'windowsphone') {

       var channel = require('cordova/channel');

       channel.createSticky('onUCAudioPluginReady');
       channel.waitForInitialization('onUCAudioPluginReady');

       channel.onCordovaReady.subscribe(function() {
           exec(onMessageFromNative, undefined, 'UCAudioPlugin', 'messageChannel', []);
           channel.initializationComplete('onUCAudioPluginReady');
       });
    }

});