cordova.define("cordova-plugin-ucvideo.UCVideoPlugin", function(require, exports, module) {
               
               var argscheck = require('cordova/argscheck'),
               utils = require('cordova/utils'),
               exec = require('cordova/exec');

var mediaObjects = {};

var UCVideoPlugin = function (successCallback, errorCallback, statusCallback) {
    this.id = utils.createUUID();
    mediaObjects[this.id] = this;
    this.successCallback = successCallback;
    this.errorCallback = errorCallback;
    this.statusCallback = statusCallback;
    this._duration = -1;
    this._position = -1;
    this._isPlaying = false;
    exec(null, this.errorCallback, "UCVideoPlugin", "create", [this.id]);
};

UCVideoPlugin.get = function(id) {
    return mediaObjects[id];
};

UCVideoPlugin.prototype.prepare = function (url, title, image, subtitleUrl, videoSize, interval, showSpeed, autoPlay) {
    exec(this.successCallback, this.errorCallback, "UCVideoPlugin", "prepare", [url, title, image, subtitleUrl, videoSize, interval, showSpeed, autoPlay]);
};

UCVideoPlugin.prototype.play = function () {
   exec(this.successCallback, this.errorCallback, "UCVideoPlugin", "play", []);
};

UCVideoPlugin.prototype.pause = function () {
   exec(this.successCallback, this.errorCallback, "UCVideoPlugin", "pause", []);
};

UCVideoPlugin.prototype.stop = function () {
   exec(this.successCallback, this.errorCallback, "UCVideoPlugin", "stop", []);
};

UCVideoPlugin.prototype.seekTo = function (position) {
    exec(this.successCallback, this.errorCallback, "UCVideoPlugin", "seekTo", [position]);
};

UCVideoPlugin.prototype.setSpeed = function (speed) {
    exec(this.successCallback, this.errorCallback, "UCVideoPlugin", "setSpeed", [speed]);
};

UCVideoPlugin.prototype.setVolume = function (volume) {
    exec(this.successCallback, this.errorCallback, "UCVideoPlugin", "setVolume", [volume]);
};

UCVideoPlugin.prototype.release = function () {
   exec(this.successCallback, this.errorCallback, "UCVideoPlugin", "release", []);
};

UCVideoPlugin.prototype.show = function () {
   exec(this.successCallback, this.errorCallback, "UCVideoPlugin", "show", []);
};

UCVideoPlugin.prototype.hide = function () {
   exec(this.successCallback, this.errorCallback, "UCVideoPlugin", "hide", []);
};

UCVideoPlugin.prototype.isPlaying = function() {
    return this._isPlaying;
};

UCVideoPlugin.prototype.getDuration = function() {
    return this._duration;
};

UCVideoPlugin.prototype.getCurrentDuration = function() {
    return this._position;
};

    UCVideoPlugin.STATE_IDLE = -1;
    UCVideoPlugin.STATE_NORMAL = 0;
    UCVideoPlugin.STATE_PREPARING = 1;
    UCVideoPlugin.STATE_PREPARING_PLAYING = 2;
    UCVideoPlugin.STATE_PREPARED = 3;
    UCVideoPlugin.STATE_PLAYING = 4;
    UCVideoPlugin.STATE_PROGRESS = 5;
    UCVideoPlugin.STATE_PAUSE = 6;
    UCVideoPlugin.STATE_SEEKPROCESSED = 7;
    UCVideoPlugin.STATE_AUTOCOMPLETE = 8;
    UCVideoPlugin.STATE_ERROR = 9;

UCVideoPlugin.onStatus = function(id, msgType, value) {
    var media = mediaObjects[id];

    console.log("send state :: " + msgType + "value:" +value);
    if (media) {

        switch(msgType) {
            case UCVideoPlugin.STATE_PROGRESS :
            case UCVideoPlugin.STATE_SEEKPROCESSED:
                media._position = value;
            break;

            case UCVideoPlugin.STATE_PREPARED :
                media._duration = value;
                media._isPlaying = false;
                break;

            case UCVideoPlugin.STATE_PLAYING :
                media._duration = value;
                media._isPlaying = true;
                break;
            case UCVideoPlugin.STATE_PAUSE :
                media._duration = value;
                media._isPlaying = false;
                break;

            case UCVideoPlugin.STATE_ERROR :
            case UCVideoPlugin.STATE_AUTOCOMPLETE :
                media._isPlaying = false;
                break;

            default :
            break;
        }

        var msg = {"type": msgType, "value": value};
        media.statusCallback(msg);
    } else if (console.error) {
        console.error("Received UCVideoPlugin.onStatus callback for unknown media :: " + id);
    }

};

    module.exports = UCVideoPlugin;

    function onMessageFromNative(msg) {
        if (msg.action == 'status') {
            UCVideoPlugin.onStatus(msg.status.id, msg.status.msgType, msg.status.value);
        } else {
            throw new Error('Unknown media action' + msg.action);
        }
    }

    if (cordova.platformId === 'android' || cordova.platformId === 'amazon-fireos' || cordova.platformId === 'windowsphone') {

       var channel = require('cordova/channel');

       channel.createSticky('onUCVideoPluginReady');
       channel.waitForInitialization('onUCVideoPluginReady');

       channel.onCordovaReady.subscribe(function() {
           exec(onMessageFromNative, undefined, 'UCVideoPlugin', 'messageChannel', []);
           channel.initializationComplete('onUCVideoPluginReady');
       });
    }

});